/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_TRANSITLANESTRUCT_H
#define DPC_IMPL_TYPE_TRANSITLANESTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "dpc/impl_type_array_uint8_t_2.h"
#include "impl_type_float.h"

namespace dpc {
struct TransitLaneStruct {
    ::UInt8 transit_type{ 0U };
    ::UInt8 transit_lane_id{ 0U };
    ::dpc::array_uint8_t_2 padding{};
    ::Float transit_lane_offset{ 0.0F };
    ::UInt8 transit_connected_lane_id{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(transit_type);
        fun(transit_lane_id);
        fun(padding);
        fun(transit_lane_offset);
        fun(transit_connected_lane_id);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(transit_type);
        fun(transit_lane_id);
        fun(padding);
        fun(transit_lane_offset);
        fun(transit_connected_lane_id);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("transit_type", transit_type);
        fun("transit_lane_id", transit_lane_id);
        fun("padding", padding);
        fun("transit_lane_offset", transit_lane_offset);
        fun("transit_connected_lane_id", transit_connected_lane_id);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("transit_type", transit_type);
        fun("transit_lane_id", transit_lane_id);
        fun("padding", padding);
        fun("transit_lane_offset", transit_lane_offset);
        fun("transit_connected_lane_id", transit_connected_lane_id);
    }

    friend bool operator==(const ::dpc::TransitLaneStruct& lhs, const ::dpc::TransitLaneStruct& rhs) noexcept
    {
        return (lhs.transit_type == rhs.transit_type) && (lhs.transit_lane_id == rhs.transit_lane_id) && (lhs.padding == rhs.padding) && (fabs(static_cast<double>(lhs.transit_lane_offset - rhs.transit_lane_offset)) < DBL_EPSILON) && (lhs.transit_connected_lane_id == rhs.transit_connected_lane_id);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_TRANSITLANESTRUCT_H
